#
# This file is part of pyasn1-modules software.
#
# Copyright (c) 2005-2020, Ilya Etingof <etingof@gmail.com>
# License: https://www.pysnmp.com/pyasn1/license.html
#
# SNMPv1 message syntax
#
# ASN.1 source from:
# http://www.ietf.org/rfc/rfc1157.txt
#
# Sample captures from:
# http://wiki.wireshark.org/SampleCaptures/
#
from pyasn1.type import namedtype, namedval, tag, univ

from pyasn1_modules import rfc1155


class Version(univ.Integer):
    namedValues = namedval.NamedValues(("version-1", 0))
    defaultValue = 0


class Community(univ.OctetString):
    pass


class RequestID(univ.Integer):
    pass


class ErrorStatus(univ.Integer):
    namedValues = namedval.NamedValues(
        ("noError", 0),
        ("tooBig", 1),
        ("noSuchName", 2),
        ("badValue", 3),
        ("readOnly", 4),
        ("genErr", 5),
    )


class ErrorIndex(univ.Integer):
    pass


class VarBind(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("name", rfc1155.ObjectName()),
        namedtype.NamedType("value", rfc1155.ObjectSyntax()),
    )


class VarBindList(univ.SequenceOf):
    componentType = VarBind()


class _RequestBase(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("request-id", RequestID()),
        namedtype.NamedType("error-status", ErrorStatus()),
        namedtype.NamedType("error-index", ErrorIndex()),
        namedtype.NamedType("variable-bindings", VarBindList()),
    )


class GetRequestPDU(_RequestBase):
    tagSet = _RequestBase.tagSet.tagImplicitly(
        tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 0)
    )


class GetNextRequestPDU(_RequestBase):
    tagSet = _RequestBase.tagSet.tagImplicitly(
        tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 1)
    )


class GetResponsePDU(_RequestBase):
    tagSet = _RequestBase.tagSet.tagImplicitly(
        tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 2)
    )


class SetRequestPDU(_RequestBase):
    tagSet = _RequestBase.tagSet.tagImplicitly(
        tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 3)
    )


class TrapPDU(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("enterprise", univ.ObjectIdentifier()),
        namedtype.NamedType("agent-addr", rfc1155.NetworkAddress()),
        namedtype.NamedType(
            "generic-trap",
            univ.Integer().clone(
                namedValues=namedval.NamedValues(
                    ("coldStart", 0),
                    ("warmStart", 1),
                    ("linkDown", 2),
                    ("linkUp", 3),
                    ("authenticationFailure", 4),
                    ("egpNeighborLoss", 5),
                    ("enterpriseSpecific", 6),
                )
            ),
        ),
        namedtype.NamedType("specific-trap", univ.Integer()),
        namedtype.NamedType("time-stamp", rfc1155.TimeTicks()),
        namedtype.NamedType("variable-bindings", VarBindList()),
    )


class Pdus(univ.Choice):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("get-request", GetRequestPDU()),
        namedtype.NamedType("get-next-request", GetNextRequestPDU()),
        namedtype.NamedType("get-response", GetResponsePDU()),
        namedtype.NamedType("set-request", SetRequestPDU()),
        namedtype.NamedType("trap", TrapPDU()),
    )


class Message(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("version", Version()),
        namedtype.NamedType("community", Community()),
        namedtype.NamedType("data", Pdus()),
    )



#
# This file is part of pyasn1-modules software.
#
# Copyright (c) 2005-2020, Ilya Etingof <etingof@gmail.com>
# License: https://www.pysnmp.com/pyasn1/license.html
#
# PKCS#1 syntax
#
# ASN.1 source from:
# ftp://ftp.rsasecurity.com/pub/pkcs/pkcs-1/pkcs-1v2.asn
#
# Sample captures could be obtained with "openssl genrsa" command
#
from pyasn1.type import namedtype, tag, univ

from pyasn1_modules.rfc2459 import AlgorithmIdentifier

pkcs_1 = univ.ObjectIdentifier("1.2.840.113549.1.1")
rsaEncryption = univ.ObjectIdentifier("1.2.840.113549.1.1.1")
md2WithRSAEncryption = univ.ObjectIdentifier("1.2.840.113549.1.1.2")
md4WithRSAEncryption = univ.ObjectIdentifier("1.2.840.113549.1.1.3")
md5WithRSAEncryption = univ.ObjectIdentifier("1.2.840.113549.1.1.4")
sha1WithRSAEncryption = univ.ObjectIdentifier("1.2.840.113549.1.1.5")
rsaOAEPEncryptionSET = univ.ObjectIdentifier("1.2.840.113549.1.1.6")
id_RSAES_OAEP = univ.ObjectIdentifier("1.2.840.113549.1.1.7")
id_mgf1 = univ.ObjectIdentifier("1.2.840.113549.1.1.8")
id_pSpecified = univ.ObjectIdentifier("1.2.840.113549.1.1.9")
id_sha1 = univ.ObjectIdentifier("1.3.14.3.2.26")

MAX = float("inf")


class Version(univ.Integer):
    pass


class RSAPrivateKey(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("version", Version()),
        namedtype.NamedType("modulus", univ.Integer()),
        namedtype.NamedType("publicExponent", univ.Integer()),
        namedtype.NamedType("privateExponent", univ.Integer()),
        namedtype.NamedType("prime1", univ.Integer()),
        namedtype.NamedType("prime2", univ.Integer()),
        namedtype.NamedType("exponent1", univ.Integer()),
        namedtype.NamedType("exponent2", univ.Integer()),
        namedtype.NamedType("coefficient", univ.Integer()),
    )


class RSAPublicKey(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("modulus", univ.Integer()),
        namedtype.NamedType("publicExponent", univ.Integer()),
    )


# XXX defaults not set
class RSAES_OAEP_params(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType(
            "hashFunc",
            AlgorithmIdentifier().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 0)
            ),
        ),
        namedtype.NamedType(
            "maskGenFunc",
            AlgorithmIdentifier().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 1)
            ),
        ),
        namedtype.NamedType(
            "pSourceFunc",
            AlgorithmIdentifier().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 2)
            ),
        ),
    )



#
# This file is part of pyasn1-modules software.
#
# Created by Russ Housley
# Copyright (c) 2019, Vigil Security, LLC
# License: https://www.pysnmp.com/pyasn1/license.html
#
import sys
import unittest

from pyasn1.codec.der.decoder import decode as der_decoder
from pyasn1.codec.der.encoder import encode as der_encoder

from pyasn1_modules import pem, rfc6402, rfc7894


class AlternativeChallengePasswordTestCase(unittest.TestCase):
    otp_pem_text = """\
MIICsjCCAZwCAQAwJDELMAkGA1UEBhMCVVMxFTATBgNVBAMTDDRUUzJWMk5MWEE2
WjCCASIwDQYJKoZIhvcNAQEBBQADggEPADCCAQoCggEBAKmF0oUj5+1rBB+pUO8X
7FPxer+1BhWOa54RTSucJmBaLx0H95qNaBCcctNDl1kcmIro/a0zMcEvj5Do29vQ
lStJdTeJ/B3X4qzOGShupxJcAhCreRZjN6Yz3T9z0zJ8OPnRvJOzcSiIzlubc9lK
Cpq4U0UsCLLfymOgL9NH4lZi96J+PFuJr0J+rTY38076U2jcPqNq5/L/d6NV9Sz2
IVOvCK1kqP/nElJVibIQZvj9YESLUKyVAfTNxLj3+IpioOOv2dT3kB9wdi4plAVi
UFEUvED1okRrI29+LdPV1UXglOCksyJIIw+DgDtutDE5Co6QkTNURFEdKIV9Sg13
zEECAwEAAaBLMBkGCyqGSIb3DQEJEAI4MQoTCDkwNTAzODQ2MC4GCSqGSIb3DQEJ
DjEhMB8wHQYDVR0OBBYEFBj12LVowM16Ed0D+AmoElKNYP/kMAsGCSqGSIb3DQEB
CwOCAQEAZZdDWKejs3UVfgZI3R9cMWGijmscVeZrjwFVkn7MI9pEDZ2aS1QaRYjY
1cu9j3i+LQp9LWPIW/ztYk11e/OcZp3fo8pZ+MT66n7YTWfDXNkqqA5xmI84DMEx
/cqenyzOBZWqpZGx7eyM9BtnrdeJ0r2qSc7LYU25FbIQFJJf8IvgMAXWMs50fvs2
Gzns447x952se2ReQ3vYhXdHvYYcgAZfSJZvK+nCmhzzqowv5p15Y5S+IHpBSXTO
a1qhNW4cjdicQZUeQ2R5kiuwZ+8vHaq9jKxAEk0hBeqG6RQaxvNOBQhHtTLNGw/C
NmaF8Y2Sl/MgvC5tjs0Ck0/r3lsoLQ==
"""

    def setUp(self):
        self.asn1Spec = rfc6402.CertificationRequest()

    def testDerCodec(self):
        substrate = pem.readBase64fromText(self.otp_pem_text)
        asn1Object, rest = der_decoder(substrate, asn1Spec=self.asn1Spec)

        self.assertFalse(rest)
        self.assertTrue(asn1Object.prettyPrint())
        self.assertEqual(substrate, der_encoder(asn1Object))
        self.assertEqual(0, asn1Object["certificationRequestInfo"]["version"])

        for attr in asn1Object["certificationRequestInfo"]["attributes"]:
            self.assertIn(attr["attrType"], rfc6402.cmcControlAttributesMap)

            av, rest = der_decoder(
                attr["attrValues"][0], rfc6402.cmcControlAttributesMap[attr["attrType"]]
            )

            self.assertFalse(rest)
            self.assertEqual(attr["attrValues"][0], der_encoder(av))

            if attr["attrType"] == rfc7894.id_aa_otpChallenge:
                self.assertEqual("90503846", av["printableString"])

    def testOpenTypes(self):
        substrate = pem.readBase64fromText(self.otp_pem_text)
        asn1Object, rest = der_decoder(
            substrate, asn1Spec=self.asn1Spec, decodeOpenTypes=True
        )

        self.assertFalse(rest)
        self.assertTrue(asn1Object.prettyPrint())
        self.assertEqual(substrate, der_encoder(asn1Object))

        for attr in asn1Object["certificationRequestInfo"]["attributes"]:
            self.assertIn(attr["attrType"], rfc6402.cmcControlAttributesMap)
            if attr["attrType"] == rfc7894.id_aa_otpChallenge:
                self.assertEqual("90503846", attr["attrValues"][0]["printableString"])


suite = unittest.TestLoader().loadTestsFromModule(sys.modules[__name__])

if __name__ == "__main__":
    result = unittest.TextTestRunner(verbosity=2).run(suite)
    sys.exit(not result.wasSuccessful())



#
# This file is part of pyasn1-modules software.
#
# Created by Russ Housley.
#
# Copyright (c) 2020, Vigil Security, LLC
# License: https://www.pysnmp.com/pyasn1/license.html
#
# CBOR Content for CMS
#
# ASN.1 source from:
# https://www.rfc-editor.org/rfc/rfc8769.txt
#

from pyasn1.type import univ

id_ct_cbor = univ.ObjectIdentifier("1.2.840.113549.1.9.16.1.44")


id_ct_cborSequence = univ.ObjectIdentifier("1.2.840.113549.1.9.16.1.45")



# This file is being contributed to pyasn1-modules software.
#
# Created by Russ Housley.
#
# Copyright (c) 2019, Vigil Security, LLC
# License: https://www.pysnmp.com/pyasn1/license.html
#
# IKEv2 Certificate Bundle
#
# ASN.1 source from:
# https://www.rfc-editor.org/rfc/rfc7296.txt

from pyasn1.type import namedtype, tag, univ

from pyasn1_modules import rfc5280


class CertificateOrCRL(univ.Choice):
    pass


CertificateOrCRL.componentType = namedtype.NamedTypes(
    namedtype.NamedType(
        "cert",
        rfc5280.Certificate().subtype(
            explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
        ),
    ),
    namedtype.NamedType(
        "crl",
        rfc5280.CertificateList().subtype(
            explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)
        ),
    ),
)


class CertificateBundle(univ.SequenceOf):
    pass


CertificateBundle.componentType = CertificateOrCRL()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pyasn1_modules']

package_data = \
{'': ['*']}

install_requires = \
['pyasn1-lextudio>=0.4.11']

setup_kwargs = {
    'name': 'pyasn1-modules-lextudio',
    'version': '0.2.9',
    'description': 'A collection of ASN.1-based protocols modules.',
    'long_description': '\nASN.1 modules for Python\n------------------------\n[![PyPI](https://img.shields.io/pypi/v/pysnmp-pyasn1-modules.svg?maxAge=2592000)](https://pypi.org/project/pysnmp-pyasn1-modules)\n[![Python Versions](https://img.shields.io/pypi/pyversions/pysnmp-pyasn1-modules.svg)](https://pypi.org/project/pysnmp-pyasn1-modules/)\n[![CI](https://github.com/pysnmp/pyasn1-modules/actions/workflows/build-test-release.yml/badge.svg)](https://github.com/pysnmp/pyasn1-modules/actions/workflows/build-test-release.yml)\n[![Coverage Status](https://img.shields.io/codecov/c/github/pysnmp/pyasn1-modules.svg)](https://codecov.io/github/pysnmp/pyasn1-modules/)\n[![GitHub license](https://img.shields.io/badge/license-BSD-blue.svg)](https://raw.githubusercontent.com/pysnmp/pyasn1-modules/master/LICENSE.txt)\n\nThe `pyasn1-modules` package contains a collection of\n[ASN.1](https://www.itu.int/rec/dologin_pub.asp?lang=e&id=T-REC-X.208-198811-W!!PDF-E&type=items)\ndata structures expressed as Python classes based on [pyasn1](https://github.com/pysnmp/pyasn1)\ndata model.\n\nIf ASN.1 module you need is not present in this collection, try using\n[Asn1ate](https://github.com/kimgr/asn1ate) tool that compiles ASN.1 documents\ninto pyasn1 code.\n\nFeedback\n--------\n\nIf something does not work as expected, \n[open an issue](https://github.com/pysnmp/pyasn1-modules/issues) at GitHub\nor post your question [on Stack Overflow](https://stackoverflow.com/questions/ask)\n \nNew modules contributions are welcome via GitHub pull requests.\n\nCopyright (c) 2005-2020, [Ilya Etingof](mailto:etingof@gmail.com).\nAll rights reserved.\n',
    'author': 'Lex Li',
    'author_email': 'support@lextudio.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



#
# This file is part of pyasn1-modules software.
#
# Created by Russ Housley with assistance from asn1ate v.0.6.0.
#
# Copyright (c) 2019, Vigil Security, LLC
# License: https://www.pysnmp.com/pyasn1/license.html
#
# An Internet Attribute Certificate Profile for Authorization
#
# ASN.1 source from:
# https://www.rfc-editor.org/rfc/rfc5275.txt
#

from pyasn1.type import constraint, namedtype, namedval, opentype, tag, univ, useful

from pyasn1_modules import rfc3565, rfc5280, rfc5652, rfc5751, rfc5755

MAX = float("inf")


# Initialize the map for GLAQueryRequests and GLAQueryResponses

glaQueryRRMap = {}


# Imports from RFC 3565

id_aes128_wrap = rfc3565.id_aes128_wrap


# Imports from RFC 5280

AlgorithmIdentifier = rfc5280.AlgorithmIdentifier

Certificate = rfc5280.Certificate

GeneralName = rfc5280.GeneralName


# Imports from RFC 5652

CertificateSet = rfc5652.CertificateSet

KEKIdentifier = rfc5652.KEKIdentifier

RecipientInfos = rfc5652.RecipientInfos


# Imports from RFC 5751

SMIMECapability = rfc5751.SMIMECapability


# Imports from RFC 5755

AttributeCertificate = rfc5755.AttributeCertificate


# The GL symmetric key distribution object identifier arc

id_skd = univ.ObjectIdentifier(
    (
        1,
        2,
        840,
        113549,
        1,
        9,
        16,
        8,
    )
)


# The GL Use KEK control attribute

id_skd_glUseKEK = id_skd + (1,)


class Certificates(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.OptionalNamedType(
            "pKC",
            Certificate().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
            ),
        ),
        namedtype.OptionalNamedType(
            "aC",
            univ.SequenceOf(componentType=AttributeCertificate())
            .subtype(subtypeSpec=constraint.ValueSizeConstraint(1, MAX))
            .subtype(implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)),
        ),
        namedtype.OptionalNamedType(
            "certPath",
            CertificateSet().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 2)
            ),
        ),
    )


class GLInfo(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("glName", GeneralName()),
        namedtype.NamedType("glAddress", GeneralName()),
    )


class GLOwnerInfo(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("glOwnerName", GeneralName()),
        namedtype.NamedType("glOwnerAddress", GeneralName()),
        namedtype.OptionalNamedType("certificates", Certificates()),
    )


class GLAdministration(univ.Integer):
    namedValues = namedval.NamedValues(("unmanaged", 0), ("managed", 1), ("closed", 2))


requested_algorithm = SMIMECapability().subtype(
    implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 4)
)
requested_algorithm["capabilityID"] = id_aes128_wrap


class GLKeyAttributes(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.DefaultedNamedType(
            "rekeyControlledByGLO",
            univ.Boolean().subtype(
                value=0,
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0),
            ),
        ),
        namedtype.DefaultedNamedType(
            "recipientsNotMutuallyAware",
            univ.Boolean().subtype(
                value=1,
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1),
            ),
        ),
        namedtype.DefaultedNamedType(
            "duration",
            univ.Integer().subtype(
                value=0,
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 2),
            ),
        ),
        namedtype.DefaultedNamedType(
            "generationCounter",
            univ.Integer().subtype(
                value=2,
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 3),
            ),
        ),
        namedtype.DefaultedNamedType("requestedAlgorithm", requested_algorithm),
    )


class GLUseKEK(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("glInfo", GLInfo()),
        namedtype.NamedType(
            "glOwnerInfo",
            univ.SequenceOf(componentType=GLOwnerInfo()).subtype(
                subtypeSpec=constraint.ValueSizeConstraint(1, MAX)
            ),
        ),
        namedtype.DefaultedNamedType(
            "glAdministration", GLAdministration().subtype(value=1)
        ),
        namedtype.OptionalNamedType("glKeyAttributes", GLKeyAttributes()),
    )


# The Delete GL control attribute

id_skd_glDelete = id_skd + (2,)


class DeleteGL(GeneralName):
    pass


# The Add GL Member control attribute

id_skd_glAddMember = id_skd + (3,)


class GLMember(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("glMemberName", GeneralName()),
        namedtype.OptionalNamedType("glMemberAddress", GeneralName()),
        namedtype.OptionalNamedType("certificates", Certificates()),
    )


class GLAddMember(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("glName", GeneralName()),
        namedtype.NamedType("glMember", GLMember()),
    )


# The Delete GL Member control attribute

id_skd_glDeleteMember = id_skd + (4,)


class GLDeleteMember(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("glName", GeneralName()),
        namedtype.NamedType("glMemberToDelete", GeneralName()),
    )


# The GL Rekey control attribute

id_skd_glRekey = id_skd + (5,)


class GLNewKeyAttributes(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.OptionalNamedType(
            "rekeyControlledByGLO",
            univ.Boolean().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
            ),
        ),
        namedtype.OptionalNamedType(
            "recipientsNotMutuallyAware",
            univ.Boolean().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)
            ),
        ),
        namedtype.OptionalNamedType(
            "duration",
            univ.Integer().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 2)
            ),
        ),
        namedtype.OptionalNamedType(
            "generationCounter",
            univ.Integer().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 3)
            ),
        ),
        namedtype.OptionalNamedType(
            "requestedAlgorithm",
            AlgorithmIdentifier().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 4)
            ),
        ),
    )


class GLRekey(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("glName", GeneralName()),
        namedtype.OptionalNamedType("glAdministration", GLAdministration()),
        namedtype.OptionalNamedType("glNewKeyAttributes", GLNewKeyAttributes()),
        namedtype.OptionalNamedType("glRekeyAllGLKeys", univ.Boolean()),
    )


# The Add and Delete GL Owner control attributes

id_skd_glAddOwner = id_skd + (6,)

id_skd_glRemoveOwner = id_skd + (7,)


class GLOwnerAdministration(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("glName", GeneralName()),
        namedtype.NamedType("glOwnerInfo", GLOwnerInfo()),
    )


# The GL Key Compromise control attribute

id_skd_glKeyCompromise = id_skd + (8,)


class GLKCompromise(GeneralName):
    pass


# The GL Key Refresh control attribute

id_skd_glkRefresh = id_skd + (9,)


class Date(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("start", useful.GeneralizedTime()),
        namedtype.OptionalNamedType("end", useful.GeneralizedTime()),
    )


class GLKRefresh(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("glName", GeneralName()),
        namedtype.NamedType(
            "dates",
            univ.SequenceOf(componentType=Date()).subtype(
                subtypeSpec=constraint.ValueSizeConstraint(1, MAX)
            ),
        ),
    )


# The GLA Query Request control attribute

id_skd_glaQueryRequest = id_skd + (11,)


class GLAQueryRequest(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("glaRequestType", univ.ObjectIdentifier()),
        namedtype.NamedType(
            "glaRequestValue",
            univ.Any(),
            openType=opentype.OpenType("glaRequestType", glaQueryRRMap),
        ),
    )


# The GLA Query Response control attribute

id_skd_glaQueryResponse = id_skd + (12,)


class GLAQueryResponse(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("glaResponseType", univ.ObjectIdentifier()),
        namedtype.NamedType(
            "glaResponseValue",
            univ.Any(),
            openType=opentype.OpenType("glaResponseType", glaQueryRRMap),
        ),
    )


# The GLA Request/Response (glaRR) arc for glaRequestType/glaResponseType

id_cmc_glaRR = univ.ObjectIdentifier(
    (
        1,
        3,
        6,
        1,
        5,
        5,
        7,
        7,
        99,
    )
)


# The Algorithm Request

id_cmc_gla_skdAlgRequest = id_cmc_glaRR + (1,)


class SKDAlgRequest(univ.Null):
    pass


# The Algorithm Response

id_cmc_gla_skdAlgResponse = id_cmc_glaRR + (2,)

SMIMECapabilities = rfc5751.SMIMECapabilities


# The control attribute to request an updated certificate to the GLA and
# the control attribute to return an updated certificate to the GLA

id_skd_glProvideCert = id_skd + (13,)

id_skd_glManageCert = id_skd + (14,)


class GLManageCert(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("glName", GeneralName()),
        namedtype.NamedType("glMember", GLMember()),
    )


# The control attribute to distribute the GL shared KEK

id_skd_glKey = id_skd + (15,)


class GLKey(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("glName", GeneralName()),
        namedtype.NamedType("glIdentifier", KEKIdentifier()),
        namedtype.NamedType("glkWrapped", RecipientInfos()),
        namedtype.NamedType("glkAlgorithm", AlgorithmIdentifier()),
        namedtype.NamedType("glkNotBefore", useful.GeneralizedTime()),
        namedtype.NamedType("glkNotAfter", useful.GeneralizedTime()),
    )


# The CMC error types

id_cet_skdFailInfo = univ.ObjectIdentifier(
    (
        1,
        3,
        6,
        1,
        5,
        5,
        7,
        15,
        1,
    )
)


class SKDFailInfo(univ.Integer):
    namedValues = namedval.NamedValues(
        ("unspecified", 0),
        ("closedGL", 1),
        ("unsupportedDuration", 2),
        ("noGLACertificate", 3),
        ("invalidCert", 4),
        ("unsupportedAlgorithm", 5),
        ("noGLONameMatch", 6),
        ("invalidGLName", 7),
        ("nameAlreadyInUse", 8),
        ("noSpam", 9),
        ("alreadyAMember", 11),
        ("notAMember", 12),
        ("alreadyAnOwner", 13),
        ("notAnOwner", 14),
    )


# Update the map for GLAQueryRequests and GLAQueryResponses

_glaQueryRRMapUpdate = {
    id_cmc_gla_skdAlgRequest: univ.Null(""),
    id_cmc_gla_skdAlgResponse: SMIMECapabilities(),
}

glaQueryRRMap.update(_glaQueryRRMapUpdate)


# Update the map for CMC control attributes; since CMS Attributes and
# CMC Controls both use 'attrType', one map is used for both

_cmcControlAttributesMapUpdate = {
    id_skd_glUseKEK: GLUseKEK(),
    id_skd_glDelete: DeleteGL(),
    id_skd_glAddMember: GLAddMember(),
    id_skd_glDeleteMember: GLDeleteMember(),
    id_skd_glRekey: GLRekey(),
    id_skd_glAddOwner: GLOwnerAdministration(),
    id_skd_glRemoveOwner: GLOwnerAdministration(),
    id_skd_glKeyCompromise: GLKCompromise(),
    id_skd_glkRefresh: GLKRefresh(),
    id_skd_glaQueryRequest: GLAQueryRequest(),
    id_skd_glaQueryResponse: GLAQueryResponse(),
    id_skd_glProvideCert: GLManageCert(),
    id_skd_glManageCert: GLManageCert(),
    id_skd_glKey: GLKey(),
}

rfc5652.cmsAttributesMap.update(_cmcControlAttributesMapUpdate)



#
# This file is part of pyasn1-modules software.
#
# Created by Russ Housley
# Copyright (c) 2019, Vigil Security, LLC
# License: https://www.pysnmp.com/pyasn1/license.html
#
import sys
import unittest

from pyasn1.codec.der.decoder import decode as der_decoder
from pyasn1.codec.der.encoder import encode as der_encoder
from pyasn1.type import univ

from pyasn1_modules import pem, rfc4476, rfc5280, rfc5755


class AttributeCertificatePolicyTestCase(unittest.TestCase):
    pem_text = """\
MIID7zCCA1gCAQEwgY+gUTBKpEgwRjEjMCEGA1UEAwwaQUNNRSBJbnRlcm1lZGlh
dGUgRUNEU0EgQ0ExCzAJBgNVBAYTAkZJMRIwEAYDVQQKDAlBQ01FIEx0ZC4CAx7N
WqE6pDgwNjETMBEGA1UEAwwKQUNNRSBFQ0RTQTELMAkGA1UEBhMCRkkxEjAQBgNV
BAoMCUFDTUUgTHRkLqBWMFSkUjBQMQswCQYDVQQGEwJVUzELMAkGA1UECAwCVkEx
EDAOBgNVBAcMB0hlcm5kb24xIjAgBgNVBAoMGUJvZ3VzIEF0dHJpYnV0ZSBBdXRo
b3RpdHkwDQYJKoZIhvcNAQELBQACBAu1MO4wIhgPMjAxOTEyMTUxMjAwMDBaGA8y
MDE5MTIzMTEyMDAwMFowgfIwPAYIKwYBBQUHCgExMDAuhgt1cm46c2VydmljZaQV
MBMxETAPBgNVBAMMCHVzZXJuYW1lBAhwYXNzd29yZDAyBggrBgEFBQcKAjEmMCSG
C3VybjpzZXJ2aWNlpBUwEzERMA8GA1UEAwwIdXNlcm5hbWUwNQYIKwYBBQUHCgMx
KTAnoBikFjAUMRIwEAYDVQQDDAlBQ01FIEx0ZC4wCwwJQUNNRSBMdGQuMCAGCCsG
AQUFBwoEMRQwEjAQDAZncm91cDEMBmdyb3VwMjAlBgNVBEgxHjANoQuGCXVybjpy
b2xlMTANoQuGCXVybjpyb2xlMjCCATkwHwYDVR0jBBgwFoAUgJCMhskAsEBzvklA
X8yJBOXO500wCQYDVR04BAIFADA8BgNVHTcENTAzoAqGCHVybjp0ZXN0oBaCFEFD
TUUtTHRkLmV4YW1wbGUuY29toA2GC3Vybjphbm90aGVyMIHMBggrBgEFBQcBDwSB
vzCBvDCBuQYKKwYBBAGBrGAwCjCBqjBFBggrBgEFBQcCBBY5aHR0cHM6Ly93d3cu
ZXhhbXBsZS5jb20vYXR0cmlidXRlLWNlcnRpZmljYXRlLXBvbGljeS5odG1sMGEG
CCsGAQUFBwIFMFUwIwwZQm9ndXMgQXR0cmlidXRlIEF1dGhvcml0eTAGAgEKAgEU
Gi5URVNUIGF0dHJpYnV0ZSBjZXJ0aWZpY2F0ZSBwb2xpY3kgZGlzcGxheSB0ZXh0
MA0GCSqGSIb3DQEBCwUAA4GBACygfTs6TkPurZQTLufcE3B1H2707OXKsJlwRpuo
dR2oJbunSHZ94jcJHs5dfbzFs6vNfVLlBiDBRieX4p+4JcQ2P44bkgyiUTJu7g1b
6C1liB3vO6yH5hOZicOAaKd+c/myuGb9uFRoaXNfc2lnbmF0dXJlX2lzX2ludmFs
aWQh
"""

    def setUp(self):
        self.asn1Spec = rfc5755.AttributeCertificate()

    def testDerCodec(self):
        substrate = pem.readBase64fromText(self.pem_text)
        asn1Object, rest = der_decoder(substrate, asn1Spec=self.asn1Spec)

        self.assertFalse(rest)
        self.assertTrue(asn1Object.prettyPrint())
        self.assertEqual(substrate, der_encoder(asn1Object))
        self.assertEqual(1, asn1Object["acinfo"]["version"])

        found_ac_policy_qualifier1 = False
        found_ac_policy_qualifier2 = False
        for extn in asn1Object["acinfo"]["extensions"]:
            self.assertIn(extn["extnID"], rfc5280.certificateExtensionsMap)
            if extn["extnID"] == rfc4476.id_pe_acPolicies:
                ev, rest = der_decoder(
                    extn["extnValue"],
                    asn1Spec=rfc5280.certificateExtensionsMap[extn["extnID"]],
                )

                self.assertFalse(rest)
                self.assertTrue(ev.prettyPrint())
                self.assertEqual(extn["extnValue"], der_encoder(ev))

                oid = univ.ObjectIdentifier(
                    (
                        1,
                        3,
                        6,
                        1,
                        4,
                        1,
                        22112,
                        48,
                        10,
                    )
                )
                self.assertEqual(oid, ev[0]["policyIdentifier"])

                for pq in ev[0]["policyQualifiers"]:
                    self.assertIn(
                        pq["policyQualifierId"], rfc5280.policyQualifierInfoMap
                    )

                    pqv, rest = der_decoder(
                        pq["qualifier"],
                        asn1Spec=rfc5280.policyQualifierInfoMap[
                            pq["policyQualifierId"]
                        ],
                    )

                    self.assertFalse(rest)
                    self.assertTrue(pqv.prettyPrint())
                    self.assertEqual(pq["qualifier"], der_encoder(pqv))

                    if pq["policyQualifierId"] == rfc4476.id_qt_acps:
                        self.assertIn("example.com", pqv)
                        found_ac_policy_qualifier1 = True

                    if pq["policyQualifierId"] == rfc4476.id_qt_acunotice:
                        self.assertIn(20, pqv[0]["noticeNumbers"])
                        found_ac_policy_qualifier2 = True

        assert found_ac_policy_qualifier1
        assert found_ac_policy_qualifier2

    def testOpenTypes(self):
        substrate = pem.readBase64fromText(self.pem_text)
        asn1Object, rest = der_decoder(
            substrate, asn1Spec=self.asn1Spec, decodeOpenTypes=True
        )

        self.assertFalse(rest)
        self.assertTrue(asn1Object.prettyPrint())
        self.assertEqual(substrate, der_encoder(asn1Object))
        self.assertEqual(1, asn1Object["acinfo"]["version"])

        found_ac_policy_qualifier1 = False
        found_ac_policy_qualifier2 = False
        for extn in asn1Object["acinfo"]["extensions"]:
            if extn["extnID"] == rfc4476.id_pe_acPolicies:
                ev, rest = der_decoder(
                    extn["extnValue"],
                    asn1Spec=rfc5280.certificateExtensionsMap[extn["extnID"]],
                    decodeOpenTypes=True,
                )

                self.assertFalse(rest)
                self.assertTrue(ev.prettyPrint())
                self.assertEqual(extn["extnValue"], der_encoder(ev))

                oid = univ.ObjectIdentifier(
                    (
                        1,
                        3,
                        6,
                        1,
                        4,
                        1,
                        22112,
                        48,
                        10,
                    )
                )
                self.assertEqual(oid, ev[0]["policyIdentifier"])

                for pq in ev[0]["policyQualifiers"]:

                    if pq["policyQualifierId"] == rfc4476.id_qt_acps:
                        self.assertIn("example.com", pq["qualifier"])
                        found_ac_policy_qualifier1 = True

                    if pq["policyQualifierId"] == rfc4476.id_qt_acunotice:
                        self.assertIn(20, pq["qualifier"][0]["noticeNumbers"])
                        found_ac_policy_qualifier2 = True

        assert found_ac_policy_qualifier1
        assert found_ac_policy_qualifier2


suite = unittest.TestLoader().loadTestsFromModule(sys.modules[__name__])

if __name__ == "__main__":
    result = unittest.TextTestRunner(verbosity=2).run(suite)
    sys.exit(not result.wasSuccessful())



#
# This file is part of pyasn1-modules software.
#
# Created by Russ Housley.
#
# Copyright (c) 2019, Vigil Security, LLC
# License: https://www.pysnmp.com/pyasn1/license.html
#
# IDEA Encryption Algorithm in CMS
#
# ASN.1 source from:
# https://www.rfc-editor.org/rfc/rfc3058.txt
# https://www.rfc-editor.org/errata/eid5913
#

from pyasn1.type import namedtype, univ

from pyasn1_modules import rfc5280

id_IDEA_CBC = univ.ObjectIdentifier("1.3.6.1.4.1.188.7.1.1.2")


id_alg_CMSIDEAwrap = univ.ObjectIdentifier("1.3.6.1.4.1.188.7.1.1.6")


class IDEA_CBCPar(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.OptionalNamedType("iv", univ.OctetString())
        # exactly 8 octets, when present
    )


# Update the Algorithm Identifier map in rfc5280.py.

_algorithmIdentifierMapUpdate = {
    id_IDEA_CBC: IDEA_CBCPar(),
    id_alg_CMSIDEAwrap: univ.Null(""),
}

rfc5280.algorithmIdentifierMap.update(_algorithmIdentifierMapUpdate)



#
# This file is part of pyasn1-modules software.
#
# Created by Russ Housley
# Copyright (c) 2019, Vigil Security, LLC
# License: https://www.pysnmp.com/pyasn1/license.html
#
import sys
import unittest

from pyasn1.codec.der.decoder import decode as der_decoder
from pyasn1.codec.der.encoder import encode as der_encoder

from pyasn1_modules import pem, rfc5280, rfc5958, rfc7914, rfc8018

# From RFC 7914, Section 13


class MultiprimeRSAPrivateKeyTestCase(unittest.TestCase):
    pem_text = """\
MIHiME0GCSqGSIb3DQEFDTBAMB8GCSsGAQQB2kcECzASBAVNb3VzZQIDEAAAAgEI
AgEBMB0GCWCGSAFlAwQBKgQQyYmguHMsOwzGMPoyObk/JgSBkJb47EWd5iAqJlyy
+ni5ftd6gZgOPaLQClL7mEZc2KQay0VhjZm/7MbBUNbqOAXNM6OGebXxVp6sHUAL
iBGY/Dls7B1TsWeGObE0sS1MXEpuREuloZjcsNVcNXWPlLdZtkSH6uwWzR0PyG/Z
+ZXfNodZtd/voKlvLOw5B3opGIFaLkbtLZQwMiGtl42AS89lZg==
"""

    def setUp(self):
        self.asn1Spec = rfc5958.EncryptedPrivateKeyInfo()

    def testDerCodec(self):
        substrate = pem.readBase64fromText(self.pem_text)
        asn1Object, rest = der_decoder(substrate, asn1Spec=self.asn1Spec)

        self.assertFalse(rest)
        self.assertTrue(asn1Object.prettyPrint())
        self.assertEqual(substrate, der_encoder(asn1Object))

        ea = asn1Object["encryptionAlgorithm"]

        self.assertEqual(rfc8018.id_PBES2, ea["algorithm"])
        self.assertIn(ea["algorithm"], rfc5280.algorithmIdentifierMap)

        params, rest = der_decoder(
            ea["parameters"], asn1Spec=rfc5280.algorithmIdentifierMap[ea["algorithm"]]
        )

        self.assertFalse(rest)
        self.assertTrue(params.prettyPrint())
        self.assertEqual(ea["parameters"], der_encoder(params))

        kdf = params["keyDerivationFunc"]

        self.assertEqual(rfc7914.id_scrypt, kdf["algorithm"])
        self.assertIn(kdf["algorithm"], rfc5280.algorithmIdentifierMap)

        kdfp, rest = der_decoder(
            kdf["parameters"], asn1Spec=rfc5280.algorithmIdentifierMap[kdf["algorithm"]]
        )

        self.assertFalse(rest)
        self.assertTrue(kdfp.prettyPrint())
        self.assertTrue(kdf["parameters"], der_encoder(kdfp))
        self.assertEqual(1048576, kdfp["costParameter"])

    def testOpenTypes(self):
        substrate = pem.readBase64fromText(self.pem_text)
        asn1Object, rest = der_decoder(
            substrate, asn1Spec=self.asn1Spec, decodeOpenTypes=True
        )

        self.assertFalse(rest)
        self.assertTrue(asn1Object.prettyPrint())
        self.assertEqual(substrate, der_encoder(asn1Object))

        ea = asn1Object["encryptionAlgorithm"]

        self.assertEqual(rfc8018.id_PBES2, ea["algorithm"])

        params = asn1Object["encryptionAlgorithm"]["parameters"]

        self.assertEqual(rfc7914.id_scrypt, params["keyDerivationFunc"]["algorithm"])

        kdfp = params["keyDerivationFunc"]["parameters"]

        self.assertEqual(1048576, kdfp["costParameter"])


suite = unittest.TestLoader().loadTestsFromModule(sys.modules[__name__])

if __name__ == "__main__":
    result = unittest.TextTestRunner(verbosity=2).run(suite)
    sys.exit(not result.wasSuccessful())



#
# This file is part of pyasn1-modules software.
#
# Created by Russ Housley with assistance from asn1ate v.0.6.0.
# Modified by Russ Housley to add maps for use with opentypes.
#
# Copyright (c) 2019, Vigil Security, LLC
# License: https://www.pysnmp.com/pyasn1/license.html
#
# Logotypes in X.509 Certificates
#
# ASN.1 source from:
# https://www.rfc-editor.org/rfc/rfc3709.txt
#

from pyasn1.type import char, constraint, namedtype, namedval, tag, univ

from pyasn1_modules import rfc5280, rfc6170

MAX = float("inf")


class HashAlgAndValue(univ.Sequence):
    pass


HashAlgAndValue.componentType = namedtype.NamedTypes(
    namedtype.NamedType("hashAlg", rfc5280.AlgorithmIdentifier()),
    namedtype.NamedType("hashValue", univ.OctetString()),
)


class LogotypeDetails(univ.Sequence):
    pass


LogotypeDetails.componentType = namedtype.NamedTypes(
    namedtype.NamedType("mediaType", char.IA5String()),
    namedtype.NamedType(
        "logotypeHash",
        univ.SequenceOf(componentType=HashAlgAndValue()).subtype(
            sizeSpec=constraint.ValueSizeConstraint(1, MAX)
        ),
    ),
    namedtype.NamedType(
        "logotypeURI",
        univ.SequenceOf(componentType=char.IA5String()).subtype(
            sizeSpec=constraint.ValueSizeConstraint(1, MAX)
        ),
    ),
)


class LogotypeAudioInfo(univ.Sequence):
    pass


LogotypeAudioInfo.componentType = namedtype.NamedTypes(
    namedtype.NamedType("fileSize", univ.Integer()),
    namedtype.NamedType("playTime", univ.Integer()),
    namedtype.NamedType("channels", univ.Integer()),
    namedtype.OptionalNamedType(
        "sampleRate",
        univ.Integer().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 3)
        ),
    ),
    namedtype.OptionalNamedType(
        "language",
        char.IA5String().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 4)
        ),
    ),
)


class LogotypeAudio(univ.Sequence):
    pass


LogotypeAudio.componentType = namedtype.NamedTypes(
    namedtype.NamedType("audioDetails", LogotypeDetails()),
    namedtype.OptionalNamedType("audioInfo", LogotypeAudioInfo()),
)


class LogotypeImageType(univ.Integer):
    pass


LogotypeImageType.namedValues = namedval.NamedValues(("grayScale", 0), ("color", 1))


class LogotypeImageResolution(univ.Choice):
    pass


LogotypeImageResolution.componentType = namedtype.NamedTypes(
    namedtype.NamedType(
        "numBits",
        univ.Integer().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)
        ),
    ),
    namedtype.NamedType(
        "tableSize",
        univ.Integer().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 2)
        ),
    ),
)


class LogotypeImageInfo(univ.Sequence):
    pass


LogotypeImageInfo.componentType = namedtype.NamedTypes(
    namedtype.DefaultedNamedType(
        "type",
        LogotypeImageType()
        .subtype(implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0))
        .subtype(value="color"),
    ),
    namedtype.NamedType("fileSize", univ.Integer()),
    namedtype.NamedType("xSize", univ.Integer()),
    namedtype.NamedType("ySize", univ.Integer()),
    namedtype.OptionalNamedType("resolution", LogotypeImageResolution()),
    namedtype.OptionalNamedType(
        "language",
        char.IA5String().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 4)
        ),
    ),
)


class LogotypeImage(univ.Sequence):
    pass


LogotypeImage.componentType = namedtype.NamedTypes(
    namedtype.NamedType("imageDetails", LogotypeDetails()),
    namedtype.OptionalNamedType("imageInfo", LogotypeImageInfo()),
)


class LogotypeData(univ.Sequence):
    pass


LogotypeData.componentType = namedtype.NamedTypes(
    namedtype.OptionalNamedType(
        "image", univ.SequenceOf(componentType=LogotypeImage())
    ),
    namedtype.OptionalNamedType(
        "audio",
        univ.SequenceOf(componentType=LogotypeAudio()).subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)
        ),
    ),
)


class LogotypeReference(univ.Sequence):
    pass


LogotypeReference.componentType = namedtype.NamedTypes(
    namedtype.NamedType(
        "refStructHash",
        univ.SequenceOf(componentType=HashAlgAndValue()).subtype(
            sizeSpec=constraint.ValueSizeConstraint(1, MAX)
        ),
    ),
    namedtype.NamedType(
        "refStructURI",
        univ.SequenceOf(componentType=char.IA5String()).subtype(
            sizeSpec=constraint.ValueSizeConstraint(1, MAX)
        ),
    ),
)


class LogotypeInfo(univ.Choice):
    pass


LogotypeInfo.componentType = namedtype.NamedTypes(
    namedtype.NamedType(
        "direct",
        LogotypeData().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 0)
        ),
    ),
    namedtype.NamedType(
        "indirect",
        LogotypeReference().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 1)
        ),
    ),
)

# Other logotype type and associated object identifiers

id_logo_background = univ.ObjectIdentifier("1.3.6.1.5.5.7.20.2")

id_logo_loyalty = univ.ObjectIdentifier("1.3.6.1.5.5.7.20.1")

id_logo_certImage = rfc6170.id_logo_certImage


class OtherLogotypeInfo(univ.Sequence):
    pass


OtherLogotypeInfo.componentType = namedtype.NamedTypes(
    namedtype.NamedType("logotypeType", univ.ObjectIdentifier()),
    namedtype.NamedType("info", LogotypeInfo()),
)


# Logotype Certificate Extension

id_pe_logotype = univ.ObjectIdentifier("1.3.6.1.5.5.7.1.12")


class LogotypeExtn(univ.Sequence):
    pass


LogotypeExtn.componentType = namedtype.NamedTypes(
    namedtype.OptionalNamedType(
        "communityLogos",
        univ.SequenceOf(componentType=LogotypeInfo()).subtype(
            explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
        ),
    ),
    namedtype.OptionalNamedType(
        "issuerLogo",
        LogotypeInfo().subtype(
            explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 1)
        ),
    ),
    namedtype.OptionalNamedType(
        "subjectLogo",
        LogotypeInfo().subtype(
            explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 2)
        ),
    ),
    namedtype.OptionalNamedType(
        "otherLogos",
        univ.SequenceOf(componentType=OtherLogotypeInfo()).subtype(
            explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 3)
        ),
    ),
)


# Map of Certificate Extension OIDs to Extensions added to the
# ones that are in rfc5280.py

_certificateExtensionsMapUpdate = {
    id_pe_logotype: LogotypeExtn(),
}

rfc5280.certificateExtensionsMap.update(_certificateExtensionsMapUpdate)



#
# This file is part of pyasn1-modules software.
#
# Copyright (c) 2005-2020, Ilya Etingof <etingof@gmail.com>
# License: https://www.pysnmp.com/pyasn1/license.html
#
# PKCS#1 syntax
#
# ASN.1 source from:
# ftp://ftp.rsasecurity.com/pub/pkcs/pkcs-1/pkcs-1v2-1.asn
#
# Sample captures could be obtained with "openssl genrsa" command
#
from pyasn1.type import constraint, namedval

from pyasn1_modules.rfc2437 import *


class OtherPrimeInfo(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("prime", univ.Integer()),
        namedtype.NamedType("exponent", univ.Integer()),
        namedtype.NamedType("coefficient", univ.Integer()),
    )


class OtherPrimeInfos(univ.SequenceOf):
    componentType = OtherPrimeInfo()
    sizeSpec = univ.SequenceOf.sizeSpec + constraint.ValueSizeConstraint(1, MAX)


class RSAPrivateKey(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType(
            "version",
            univ.Integer(
                namedValues=namedval.NamedValues(("two-prime", 0), ("multi", 1))
            ),
        ),
        namedtype.NamedType("modulus", univ.Integer()),
        namedtype.NamedType("publicExponent", univ.Integer()),
        namedtype.NamedType("privateExponent", univ.Integer()),
        namedtype.NamedType("prime1", univ.Integer()),
        namedtype.NamedType("prime2", univ.Integer()),
        namedtype.NamedType("exponent1", univ.Integer()),
        namedtype.NamedType("exponent2", univ.Integer()),
        namedtype.NamedType("coefficient", univ.Integer()),
        namedtype.OptionalNamedType("otherPrimeInfos", OtherPrimeInfos()),
    )



#
# This file is part of pyasn1-modules software.
#
# Copyright (c) 2019, Vigil Security, LLC
# License: https://www.pysnmp.com/pyasn1/license.html
#
import sys
import unittest

from pyasn1.codec.der.decoder import decode as der_decoder
from pyasn1.codec.der.encoder import encode as der_encoder
from pyasn1.type import univ

from pyasn1_modules import pem, rfc4357, rfc5280, rfc5652


class SignedTestCase(unittest.TestCase):
    signed_pem_text = """\
MIIBKAYJKoZIhvcNAQcCoIIBGTCCARUCAQExDDAKBgYqhQMCAgkFADAbBgkqhkiG
9w0BBwGgDgQMc2FtcGxlIHRleHQKMYHkMIHhAgEBMIGBMG0xHzAdBgNVBAMMFkdv
c3RSMzQxMC0yMDAxIGV4YW1wbGUxEjAQBgNVBAoMCUNyeXB0b1BybzELMAkGA1UE
BhMCUlUxKTAnBgkqhkiG9w0BCQEWGkdvc3RSMzQxMC0yMDAxQGV4YW1wbGUuY29t
AhAr9cYewhG9F8fc1GJmtC4hMAoGBiqFAwICCQUAMAoGBiqFAwICEwUABEDAw0LZ
P4/+JRERiHe/icPbg0IE1iD5aCqZ9v4wO+T0yPjVtNr74caRZzQfvKZ6DRJ7/RAl
xlHbjbL0jHF+7XKp
"""

    def setUp(self):
        self.asn1Spec = rfc5652.ContentInfo()

    def testDerCodec(self):
        substrate = pem.readBase64fromText(self.signed_pem_text)
        asn1Object, rest = der_decoder(substrate, asn1Spec=self.asn1Spec)
        self.assertFalse(rest)
        self.assertTrue(asn1Object.prettyPrint())
        self.assertEqual(substrate, der_encoder(asn1Object))
        self.assertEqual(rfc5652.id_signedData, asn1Object["contentType"])

        sd, rest = der_decoder(asn1Object["content"], asn1Spec=rfc5652.SignedData())
        self.assertFalse(rest)
        self.assertTrue(sd.prettyPrint())
        self.assertEqual(asn1Object["content"], der_encoder(sd))

        encoded_null = der_encoder(univ.Null(""))

        si = sd["signerInfos"][0]
        self.assertEqual(rfc4357.id_GostR3411_94, si["digestAlgorithm"]["algorithm"])
        self.assertEqual(encoded_null, si["digestAlgorithm"]["parameters"])

        self.assertEqual(
            rfc4357.id_GostR3410_2001, si["signatureAlgorithm"]["algorithm"]
        )
        self.assertEqual(encoded_null, si["signatureAlgorithm"]["parameters"])
        self.assertEqual(64, len(si["signature"]))

    def testOpenTypes(self):
        substrate = pem.readBase64fromText(self.signed_pem_text)
        asn1Object, rest = der_decoder(
            substrate, asn1Spec=self.asn1Spec, decodeOpenTypes=True
        )
        self.assertFalse(rest)
        self.assertTrue(asn1Object.prettyPrint())
        self.assertEqual(substrate, der_encoder(asn1Object))
        self.assertEqual(rfc5652.id_signedData, asn1Object["contentType"])

        si = asn1Object["content"]["signerInfos"][0]
        self.assertEqual(rfc4357.id_GostR3411_94, si["digestAlgorithm"]["algorithm"])
        self.assertEqual(univ.Null(""), si["digestAlgorithm"]["parameters"])

        self.assertEqual(
            rfc4357.id_GostR3410_2001, si["signatureAlgorithm"]["algorithm"]
        )
        self.assertEqual(univ.Null(""), si["signatureAlgorithm"]["parameters"])

        self.assertEqual(64, len(si["signature"]))


class KeyAgreeTestCase(unittest.TestCase):
    keyagree_pem_text = """\
MIIBpAYJKoZIhvcNAQcDoIIBlTCCAZECAQIxggFQoYIBTAIBA6BloWMwHAYGKoUD
AgITMBIGByqFAwICJAAGByqFAwICHgEDQwAEQLNVOfRngZcrpcTZhB8n+4HtCDLm
mtTyAHi4/4Nk6tIdsHg8ff4DwfQG5DvMFrnF9vYZNxwXuKCqx9GhlLOlNiChCgQI
L/D20YZLMoowHgYGKoUDAgJgMBQGByqFAwICDQAwCQYHKoUDAgIfATCBszCBsDCB
gTBtMR8wHQYDVQQDDBZHb3N0UjM0MTAtMjAwMSBleGFtcGxlMRIwEAYDVQQKDAlD
cnlwdG9Qcm8xCzAJBgNVBAYTAlJVMSkwJwYJKoZIhvcNAQkBFhpHb3N0UjM0MTAt
MjAwMUBleGFtcGxlLmNvbQIQK/XGHsIRvRfH3NRiZrQuIQQqMCgEIBajHOfOTukN
8ex0aQRoHsefOu24Ox8dSn75pdnLGdXoBAST/YZ+MDgGCSqGSIb3DQEHATAdBgYq
hQMCAhUwEwQItzXhegc1oh0GByqFAwICHwGADDmxivS/qeJlJbZVyQ==
"""

    def setUp(self):
        self.asn1Spec = rfc5652.ContentInfo()

    def testDerCodec(self):
        substrate = pem.readBase64fromText(self.keyagree_pem_text)
        asn1Object, rest = der_decoder(substrate, asn1Spec=self.asn1Spec)
        self.assertFalse(rest)
        self.assertTrue(asn1Object.prettyPrint())
        self.assertEqual(substrate, der_encoder(asn1Object))
        self.assertEqual(rfc5652.id_envelopedData, asn1Object["contentType"])

        ed, rest = der_decoder(asn1Object["content"], asn1Spec=rfc5652.EnvelopedData())
        self.assertFalse(rest)
        self.assertTrue(ed.prettyPrint())
        self.assertEqual(asn1Object["content"], der_encoder(ed))

        ri = ed["recipientInfos"][0]
        alg1 = ri["kari"]["originator"]["originatorKey"]["algorithm"]
        self.assertEqual(rfc4357.id_GostR3410_2001, alg1["algorithm"])
        param1, rest = der_decoder(
            alg1["parameters"], asn1Spec=rfc4357.GostR3410_2001_PublicKeyParameters()
        )
        self.assertFalse(rest)
        self.assertTrue(param1.prettyPrint())
        self.assertEqual(alg1["parameters"], der_encoder(param1))

        self.assertEqual(
            rfc4357.id_GostR3410_2001_CryptoPro_XchA_ParamSet,
            param1["publicKeyParamSet"],
        )
        self.assertEqual(
            rfc4357.id_GostR3411_94_CryptoProParamSet, param1["digestParamSet"]
        )

        self.assertEqual(8, len(ri["kari"]["ukm"]))

        alg2 = ed["encryptedContentInfo"]["contentEncryptionAlgorithm"]
        self.assertEqual(
            rfc4357.id_Gost28147_89,
            alg2["algorithm"],
        )
        param2, rest = der_decoder(
            alg2["parameters"], asn1Spec=rfc4357.Gost28147_89_Parameters()
        )
        self.assertFalse(rest)
        self.assertTrue(param1.prettyPrint())
        self.assertEqual(alg2["parameters"], der_encoder(param2))

        self.assertEqual(8, len(param2["iv"]))
        self.assertEqual(
            rfc4357.id_Gost28147_89_CryptoPro_A_ParamSet, param2["encryptionParamSet"]
        )

    def testOpenTypes(self):
        openTypeMap = {
            rfc4357.id_GostR3410_2001: rfc4357.GostR3410_2001_PublicKeyParameters(),
            rfc4357.id_Gost28147_89: rfc4357.Gost28147_89_Parameters(),
        }

        substrate = pem.readBase64fromText(self.keyagree_pem_text)
        asn1Object, rest = der_decoder(
            substrate,
            asn1Spec=self.asn1Spec,
            openTypes=openTypeMap,
            decodeOpenTypes=True,
        )
        self.assertFalse(rest)
        self.assertTrue(asn1Object.prettyPrint())
        self.assertEqual(substrate, der_encoder(asn1Object))
        self.assertEqual(rfc5652.id_envelopedData, asn1Object["contentType"])

        ri = asn1Object["content"]["recipientInfos"][0]
        alg1 = ri["kari"]["originator"]["originatorKey"]["algorithm"]
        self.assertEqual(rfc4357.id_GostR3410_2001, alg1["algorithm"])
        param1 = alg1["parameters"]
        self.assertEqual(
            rfc4357.id_GostR3410_2001_CryptoPro_XchA_ParamSet,
            param1["publicKeyParamSet"],
        )
        self.assertEqual(
            rfc4357.id_GostR3411_94_CryptoProParamSet, param1["digestParamSet"]
        )

        self.assertEqual(8, len(ri["kari"]["ukm"]))

        alg2 = asn1Object["content"]["encryptedContentInfo"][
            "contentEncryptionAlgorithm"
        ]
        self.assertEqual(rfc4357.id_Gost28147_89, alg2["algorithm"])
        param2 = alg2["parameters"]
        self.assertEqual(8, len(param2["iv"]))
        self.assertEqual(
            rfc4357.id_Gost28147_89_CryptoPro_A_ParamSet, param2["encryptionParamSet"]
        )


class KeyTransportTestCase(unittest.TestCase):
    keytrans_pem_text = """\
MIIBpwYJKoZIhvcNAQcDoIIBmDCCAZQCAQAxggFTMIIBTwIBADCBgTBtMR8wHQYD
VQQDDBZHb3N0UjM0MTAtMjAwMSBleGFtcGxlMRIwEAYDVQQKDAlDcnlwdG9Qcm8x
CzAJBgNVBAYTAlJVMSkwJwYJKoZIhvcNAQkBFhpHb3N0UjM0MTAtMjAwMUBleGFt
cGxlLmNvbQIQK/XGHsIRvRfH3NRiZrQuITAcBgYqhQMCAhMwEgYHKoUDAgIkAAYH
KoUDAgIeAQSBpzCBpDAoBCBqL6ghBpVon5/kR6qey2EVK35BYLxdjfv1PSgbGJr5
dQQENm2Yt6B4BgcqhQMCAh8BoGMwHAYGKoUDAgITMBIGByqFAwICJAAGByqFAwIC
HgEDQwAEQE0rLzOQ5tyj3VUqzd/g7/sx93N+Tv+/eImKK8PNMZQESw5gSJYf28dd
Em/askCKd7W96vLsNMsjn5uL3Z4SwPYECJeV4ywrrSsMMDgGCSqGSIb3DQEHATAd
BgYqhQMCAhUwEwQIvBCLHwv/NCkGByqFAwICHwGADKqOch3uT7Mu4w+hNw==
"""

    def setUp(self):
        self.asn1Spec = rfc5652.ContentInfo()

    def testDerCodec(self):
        substrate = pem.readBase64fromText(self.keytrans_pem_text)
        asn1Object, rest = der_decoder(substrate, asn1Spec=self.asn1Spec)
        self.assertFalse(rest)
        self.assertTrue(asn1Object.prettyPrint())
        self.assertEqual(substrate, der_encoder(asn1Object))
        self.assertEqual(rfc5652.id_envelopedData, asn1Object["contentType"])

        ed, rest = der_decoder(asn1Object["content"], asn1Spec=rfc5652.EnvelopedData())
        self.assertFalse(rest)
        self.assertTrue(ed.prettyPrint())
        self.assertEqual(asn1Object["content"], der_encoder(ed))

        ri = ed["recipientInfos"][0]
        alg1 = ri["ktri"]["keyEncryptionAlgorithm"]
        self.assertEqual(rfc4357.id_GostR3410_2001, alg1["algorithm"])
        param1, rest = der_decoder(
            alg1["parameters"], asn1Spec=rfc4357.GostR3410_2001_PublicKeyParameters()
        )
        self.assertFalse(rest)
        self.assertTrue(param1.prettyPrint())
        self.assertEqual(alg1["parameters"], der_encoder(param1))
        self.assertEqual(
            rfc4357.id_GostR3410_2001_CryptoPro_XchA_ParamSet,
            param1["publicKeyParamSet"],
        )
        self.assertEqual(
            rfc4357.id_GostR3411_94_CryptoProParamSet, param1["digestParamSet"]
        )

        alg2 = ed["encryptedContentInfo"]["contentEncryptionAlgorithm"]
        self.assertEqual(rfc4357.id_Gost28147_89, alg2["algorithm"])
        param2, rest = der_decoder(
            alg2["parameters"], asn1Spec=rfc4357.Gost28147_89_Parameters()
        )
        self.assertFalse(rest)
        self.assertTrue(param2.prettyPrint())
        self.assertEqual(alg2["parameters"], der_encoder(param2))
        self.assertEqual(8, len(param2["iv"]))
        self.assertEqual(
            rfc4357.id_Gost28147_89_CryptoPro_A_ParamSet, param2["encryptionParamSet"]
        )

    def testOpenTypes(self):
        openTypeMap = {
            rfc4357.id_GostR3410_2001: rfc4357.GostR3410_2001_PublicKeyParameters(),
            rfc4357.id_Gost28147_89: rfc4357.Gost28147_89_Parameters(),
        }

        substrate = pem.readBase64fromText(self.keytrans_pem_text)
        asn1Object, rest = der_decoder(
            substrate,
            asn1Spec=self.asn1Spec,
            openTypes=openTypeMap,
            decodeOpenTypes=True,
        )
        self.assertFalse(rest)
        self.assertTrue(asn1Object.prettyPrint())
        self.assertEqual(substrate, der_encoder(asn1Object))

        ri = asn1Object["content"]["recipientInfos"][0]
        alg1 = ri["ktri"]["keyEncryptionAlgorithm"]
        self.assertEqual(rfc4357.id_GostR3410_2001, alg1["algorithm"])
        param1 = alg1["parameters"]
        self.assertEqual(
            rfc4357.id_GostR3410_2001_CryptoPro_XchA_ParamSet,
            param1["publicKeyParamSet"],
        )
        self.assertEqual(
            rfc4357.id_GostR3411_94_CryptoProParamSet, param1["digestParamSet"]
        )

        alg2 = asn1Object["content"]["encryptedContentInfo"][
            "contentEncryptionAlgorithm"
        ]
        self.assertEqual(rfc4357.id_Gost28147_89, alg2["algorithm"])
        param2 = alg2["parameters"]
        self.assertEqual(8, len(param2["iv"]))
        self.assertEqual(
            rfc4357.id_Gost28147_89_CryptoPro_A_ParamSet, param2["encryptionParamSet"]
        )


suite = unittest.TestLoader().loadTestsFromModule(sys.modules[__name__])

if __name__ == "__main__":
    result = unittest.TextTestRunner(verbosity=2).run(suite)
    sys.exit(not result.wasSuccessful())



#
# This file is part of pyasn1-modules software.
#
# Created by Russ Housley with assistance from asn1ate v.0.6.0.
#
# Copyright (c) 2019, Vigil Security, LLC
# License: https://www.pysnmp.com/pyasn1/license.html
#
# KEA and SKIPJACK Algorithms in CMS
#
# ASN.1 source from:
# https://www.rfc-editor.org/rfc/rfc2876.txt
#

from pyasn1.type import namedtype, univ

from pyasn1_modules import rfc5280, rfc5751

id_fortezzaConfidentialityAlgorithm = univ.ObjectIdentifier("2.16.840.1.101.2.1.1.4")


id_fortezzaWrap80 = univ.ObjectIdentifier("2.16.840.1.101.2.1.1.23")


id_kEAKeyEncryptionAlgorithm = univ.ObjectIdentifier("2.16.840.1.101.2.1.1.24")


id_keyExchangeAlgorithm = univ.ObjectIdentifier("2.16.840.1.101.2.1.1.22")


class Skipjack_Parm(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("initialization-vector", univ.OctetString())
    )


# Update the Algorithm Identifier map in rfc5280.py.

_algorithmIdentifierMapUpdate = {
    id_fortezzaConfidentialityAlgorithm: Skipjack_Parm(),
    id_kEAKeyEncryptionAlgorithm: rfc5280.AlgorithmIdentifier(),
}

rfc5280.algorithmIdentifierMap.update(_algorithmIdentifierMapUpdate)


# Update the SMIMECapabilities Attribute map in rfc5751.py

_smimeCapabilityMapUpdate = {
    id_kEAKeyEncryptionAlgorithm: rfc5280.AlgorithmIdentifier(),
}

rfc5751.smimeCapabilityMap.update(_smimeCapabilityMapUpdate)
